var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { outputFileSync, removeSync, readFileSync, existsSync, readFile, pathExists } from "fs-extra";
import * as traverse from "filewalker";
import * as watch from "node-watch";
import { SvelteTranscriber } from "../src";
import { config } from "./config";
import { Transcriber } from "typedraft";
import { transformSync, transformAsync } from "@babel/core";
import * as TypescriptPreset from "@babel/preset-typescript";
function TraverseDirectory(path, callback) {
    const action = (relative, stats, absolute) => callback(relative, absolute);
    traverse(path)
        .on("file", action)
        .walk();
}
export function InspectDirectory(path) {
    ComposeDirectory(path);
    //@ts-ignore
    watch(path, { recursive: true }, (event, name) => {
        if (name.endsWith(".tsx")) {
            console.log(event, name);
            try {
                ComposeFile(name);
            }
            catch (error) {
                console.log(error.message);
            }
        }
    });
}
export function InspectFile(path) {
    ComposeFile(path);
    //@ts-ignore
    watch(path, (event, name) => {
        if (name.endsWith(".tsx")) {
            console.log(event, name);
            try {
                ComposeFile(name);
            }
            catch (error) {
                console.log(error.message);
            }
        }
    });
}
export function ComposeDirectory(path) {
    TraverseDirectory(path, (relative, absolute) => {
        if (absolute.endsWith(".tsx")) {
            try {
                ComposeFile(absolute);
            }
            catch (error) {
                console.log(`compose file failed: ${error.message}, source: ${relative}`);
            }
        }
    });
}
export function CrossoutDirectory(path) {
    TraverseDirectory(path, (relative, absolute) => {
        if (absolute.endsWith(".tsx")) {
            removeSync(absolute.replace(".tsx", ""));
        }
    });
}
export function ComposeFile(source) {
    if (source.endsWith(".js.tsx") || source.endsWith(".ts")) {
        const code = TranscribeTypeDraftSync(source);
        outputFileSync(source.replace(source.endsWith(".js.tsx") ? ".js.tsx" : ".ts", ".js"), code, "utf8");
    }
    else if (source.endsWith(".tsx")) {
        const component = TranscribeSvelteDraftSync(source);
        outputFileSync(source.replace(".tsx", ".svelte"), component, "utf8");
    }
}
export function TranscribeTypeDraftAsync(source) {
    return __awaiter(this, void 0, void 0, function* () {
        const code = yield readFile(source, "utf8");
        const transcriber = new Transcriber(code);
        config.dsls.forEach(dsl => transcriber.AddDSL(dsl.name, dsl.dsl));
        const ts_code = transcriber.Transcribe();
        const js_code = yield transformAsync(ts_code, {
            filename: "script.tsx",
            ast: true,
            presets: [[TypescriptPreset, { jsxPragma: "preserve", isTSX: true, allExtensions: true }]]
        });
        return js_code.code;
    });
}
export function TranscribeTypeDraftSync(source) {
    const code = readFileSync(source, "utf8");
    const transcriber = new Transcriber(code);
    config.dsls.forEach(dsl => transcriber.AddDSL(dsl.name, dsl.dsl));
    const ts_code = transcriber.Transcribe();
    const js_code = transformSync(ts_code, {
        filename: "script.tsx",
        ast: true,
        presets: [[TypescriptPreset, { jsxPragma: "preserve", isTSX: true, allExtensions: true }]]
    }).code;
    return js_code;
}
export function TranscribeSvelteDraftAsync(source) {
    return __awaiter(this, void 0, void 0, function* () {
        //
        const code = yield readFile(source, "utf8");
        const { import_section, script_section, template_section, module_context } = Transcribe(code);
        //
        const style_path = source.replace(".tsx", ".css");
        const style_section = (yield pathExists(style_path)) ? yield readFile(style_path, "utf8") : "";
        //
        const component = AssembleComponent(import_section, script_section, template_section, style_section, module_context);
        return component;
    });
}
export function TranscribeSvelteDraftSync(source) {
    //
    const code = readFileSync(source, "utf8");
    const { import_section, script_section, template_section, module_context } = Transcribe(code);
    //
    const style = source.replace(".tsx", ".css");
    const style_section = existsSync(style) ? readFileSync(style, "utf8") : "";
    //
    const component = AssembleComponent(import_section, script_section, template_section, style_section, module_context);
    return component;
}
function Transcribe(code) {
    const transcriber = new SvelteTranscriber(code);
    config.dsls.forEach(dsl => transcriber.AddDSL(dsl.name, dsl.dsl));
    const module_context = transcriber.ExtractModuleContext();
    const { import_section, script_section, template_section } = transcriber.TranscribeToSections();
    return { import_section, script_section, template_section, module_context };
}
function AssembleComponent(import_section, script_section, template_section, style_section, module_context) {
    const module_context_section = module_context ? `<script context="module">\n${module_context}\n</script>` : "";
    const component = [
        module_context_section,
        "<script>",
        import_section,
        "\n",
        script_section,
        "</script>",
        "\n",
        template_section,
        "\n",
        "<style>",
        style_section,
        "</style>"
    ].join("\n");
    return component;
}
